import { Component, Project } from 'projen';
import { AwsCdkTypeScriptApp } from 'projen/lib/awscdk';
import { AwsCdkDeps } from 'projen/lib/awscdk/awscdk-deps';
import { AutoDiscoverBase } from 'projen/lib/cdk';
/**
 * For future use. No properties, yet.
 */
export interface StepFunctionsAutoDiscoverOptions {
}
/**
 * A projen component for discovering AWS Step Function state machine workflow ASL files
 * and generating a strongly typed interface and construct to use it.
 *
 * Simply add a new instance and hand it your AwsCdkTypeScriptApp projen class:
 * ```
 * const project = new AwsCdkTypeScriptApp({ ... });
 * new StepFunctionsAutoDiscover(project);
 * ```
 *
 * And any *.workflow.json file will cause the generation of a new strongly-typed StateMachine-derived class you can use.
 * Note that these constructs are NOT jsii-compatible. If you need that,
 * please open an [issue](https://github.com/mbonig/state-machine/issues/new)
 */
export declare class StepFunctionsAutoDiscover extends AutoDiscoverBase {
    constructor(project: AwsCdkTypeScriptApp, _options?: StepFunctionsAutoDiscoverOptions);
}
export interface StepFunctionsStateMachineOptions {
    readonly constructFile?: string;
    readonly workflowAsl: string;
    readonly constructName?: string;
    readonly cdkDeps: AwsCdkDeps;
}
/**
 * Don't use this class directly.
 */
export declare class StepFunctionsStateMachine extends Component {
    constructor(project: Project, options: StepFunctionsStateMachineOptions);
}
